﻿using System;
using System.Drawing;
using System.Collections.Generic;
using System.Collections;
using System.Collections.Specialized;
using System.Data;
using System.Data.SqlClient;
using System.Data.Common;
using System.Linq.Expressions;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using WeifenLuo.WinFormsUI.Docking;
using PPL_Lib;
using MigraDoc;
using MD = MigraDoc.DocumentObjectModel;
using MDT = MigraDoc.DocumentObjectModel.Tables;

namespace OCalcProReporting
{
    public partial class Report
    {

        void GetNotes(PPLElement pElem, List<PPLNotes> pNotes)
        {
            if (pElem is PPLNotes) pNotes.Add(pElem as PPLNotes);
            foreach (PPLElement elem in pElem.Children) GetNotes(elem, pNotes);
        }

        private String ScrubLabel(String pLabel)
        {
            bool prevWasCap = true;
            StringBuilder result = new StringBuilder();
            foreach (char c in pLabel)
            {
                if (c == ' ' || c == '_')
                {
                    result.Append(' ');
                    prevWasCap = true;
                }
                else if (c >= 'A' && c <= 'Z')
                {
                    if (!prevWasCap) result.Append(' ');
                    result.Append(c);
                    prevWasCap = true;
                }
                else
                {
                    result.Append(c);
                    prevWasCap = false;
                }
            }
            return result.ToString();
        }

        private String PoleImage()
        {
            String fname = System.IO.Path.GetTempPath() + "OCalc_Pole_Image.jpg";
            try
            {
                PPL_Lib.PPLMain.FileDelete(fname);
            }
            catch { }
            int imgIdx = 0;
            int width = 480;
            int height = 480;
            int imageWidth = width;
            int imageHeight = height;
            System.Drawing.RectangleF rect = new System.Drawing.RectangleF(0, 0, width, height);

            PPL_Lib.PPLPole pole = cPPLMain.GetPole();
            if (pole != null)
            {
                if (pole.cImages.cImages.Count > imgIdx)
                {
                    PPL_Lib.PPLPole.PoleImageSet.PoleImage imagePath = pole.cImages.cImages[imgIdx];
                    using (System.Drawing.Image image = PPL_CorvusImageViewer.CorvusBitmap.FromFile(imagePath.cImagePath))
                    {
                        int idx = imagePath.cImagePath.LastIndexOf('.');
                        String imclPath = imagePath.cImagePath.Substring(0, idx) + ".imcl";
                        if (System.IO.File.Exists(imclPath))
                        {
                            try
                            {
                                using (System.IO.StreamReader sr = new System.IO.StreamReader(imclPath))
                                {
                                    String sline = sr.ReadLine();
                                    String[] points = sline.Split(',');
                                    if (points.Length == 6 || points.Length == 8)
                                    {
                                        System.Drawing.PointF p_upper = new System.Drawing.PointF();
                                        System.Drawing.PointF p_middle = new System.Drawing.PointF();
                                        System.Drawing.PointF p_lower = new System.Drawing.PointF();
                                        p_upper.X = (float)Convert.ToDouble(points[0]);
                                        p_upper.Y = (float)Convert.ToDouble(points[1]);
                                        p_middle.X = (float)Convert.ToDouble(points[2]);
                                        p_middle.Y = (float)Convert.ToDouble(points[3]);
                                        p_lower.X = (float)Convert.ToDouble(points[4]);
                                        p_lower.Y = (float)Convert.ToDouble(points[5]);

                                        int cRotation = 0;
                                        if (p_upper.X < 0)
                                        {
                                            cRotation = 90;
                                        }
                                        else if (p_upper.Y < 0)
                                        {
                                            cRotation = 180;
                                        }
                                        else if (p_lower.X < 0)
                                        {
                                            cRotation = 270;
                                        }
                                        switch (cRotation)
                                        {
                                            case 90:
                                                image.RotateFlip(System.Drawing.RotateFlipType.Rotate90FlipNone);
                                                break;
                                            case 180:
                                                image.RotateFlip(System.Drawing.RotateFlipType.Rotate180FlipNone);
                                                break;
                                            case 270:
                                                image.RotateFlip(System.Drawing.RotateFlipType.Rotate270FlipNone);
                                                break;
                                            default:
                                                break;
                                        }
                                    }
                                }
                            }
                            catch { }
                        }
                        using (System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap((int)rect.Width, (int)rect.Height))
                        {
                            try
                            {
                                using (System.Drawing.Graphics gr = System.Drawing.Graphics.FromImage((System.Drawing.Image)bitmap))
                                {
                                    if (image != null)
                                    {
                                        gr.DrawImage(image, 0, 0, width, height);
                                        bitmap.Save(fname, System.Drawing.Imaging.ImageFormat.Jpeg);
                                    }
                                }
                            }
                            catch
                            {
                                return String.Empty;
                            }
                        }
                    }
                }
                else
                {
                    return String.Empty;
                }
            }

            if (File.Exists(fname))
            {
                return fname;
            }

            return String.Empty;
        }

        #region TABLE_ITEM_DEF
        private class CellDefinition
        {
            public CellDefinition(int pWidthInCells)
            {
                cWidthInCells = pWidthInCells;
                System.Diagnostics.Debug.Assert(pWidthInCells > 0);
            }

            public String cText = String.Empty;
            public int cWidthInCells = 1;

            public MD.Font cFont = new MD.Font(ReportFont.cFontName, MD.Unit.FromPoint(10));

            public MD.Color cTextColor = MD.Colors.Black;
            public MD.Color cBackgroundColor = MD.Colors.White;

            public bool cFontBold = false;
            public bool cFontUnderline = false;
            public bool cFontItalic = false;

            public MD.ParagraphAlignment cAllignment = MD.ParagraphAlignment.Left;
        }

        private CellDefinition Cell(params object[] pParams)
        {
            CellDefinition item = new CellDefinition(1);
            foreach (object obj in pParams)
            {
#if DEBUG
                bool widthSet = false;
                bool textSet = false;
                bool backColorSet = false;
                bool alignSet = false;
                bool fontSet = false;
                bool fontModSet = false;
                bool textColorSet = false;
#endif
                if (obj is int)
                {
#if DEBUG
                    System.Diagnostics.Debug.Assert(!widthSet, "Cell width already Set");
                    widthSet = true;
#endif
                    item.cWidthInCells = (int)obj;
                }
                else if (obj is String)
                {
#if DEBUG
                    System.Diagnostics.Debug.Assert(!textSet, "Text already Set");
                    textSet = true;
#endif
                    double dval = 0;
                    String sval = (String)obj;
                    if (sval.Length > 8)
                    {
                        if (double.TryParse(sval, out dval))
                        {
                            if (dval < 0)
                            {
                                sval = dval.ToString("0.0e+0");
                            }
                            else
                            {
                                sval = dval.ToString("0.00e+0");
                            }
                        }
                    }
                    item.cText = sval;
                }
                else if (obj is MD.Color)
                {
#if DEBUG
                    System.Diagnostics.Debug.Assert(!backColorSet, "Already Set");
                    backColorSet = true;
#endif
                    item.cBackgroundColor = (MD.Color)obj;
                }
                else if (obj is MD.ParagraphAlignment)
                {
#if DEBUG
                    System.Diagnostics.Debug.Assert(!alignSet, "Alignment already Set");
                    alignSet = true;
#endif
                    item.cAllignment = (MD.ParagraphAlignment)obj;
                }
                else if (obj is MD.Font)
                {
#if DEBUG
                    System.Diagnostics.Debug.Assert(!fontSet, "Font already Set");
                    fontSet = true;
#endif
                    item.cFont = (MD.Font)obj;
                }
                else if (obj is FontModifier)
                {
#if DEBUG
                    System.Diagnostics.Debug.Assert(!fontModSet, "Font modifier already Set");
                    fontModSet = true;
#endif
                    FontModifier pFontMod = (FontModifier)obj;
                    item.cFontBold = pFontMod.cFontBold;
                    item.cFontUnderline = pFontMod.cFontUnderline;
                    item.cFontItalic = pFontMod.cFontUnderline;
                }
                else if (obj is TextColor)
                {
#if DEBUG
                    System.Diagnostics.Debug.Assert(!textColorSet, "Text color already Set");
                    textColorSet = true;
#endif
                    item.cTextColor = ((TextColor)obj).cColor;
                }
#if DEBUG
                else
                {
                    System.Diagnostics.Debug.Assert(false, "Unexpected parameter type");
                }
#endif
            }
            return item;
        }

        private class FontModifier
        {
            public bool cFontBold = false;
            public bool cFontUnderline = false;
            public bool cFontItalic = false;
            public FontModifier(bool pFontBold, bool pFontUnderline, bool pFontItalic)
            {
                cFontBold = pFontBold;
                cFontUnderline = pFontUnderline;
                cFontItalic = pFontItalic;
            }
        }

        private class TextColor
        {
            public MD.Color cColor = MD.Colors.Black;
            public TextColor(MD.Color pColor)
            {
                cColor = pColor;
            }
        }

        #endregion

        private void CreateTable(double pWidth, int pColumns)
        {
            cCurrentTable = cDocument.LastSection.AddTable();

            for (int i = 0; i < pColumns; i++)
            {
                cCurrentTable.AddColumn(new MD.Unit(pWidth / pColumns, MD.UnitType.Inch));
            }
        }

        private void FinalizeTable(int pKeepWith = 20)
        {
            cCurrentTable.Rows[0].KeepWith = Math.Min(pKeepWith, cCurrentTable.Rows.Count - 1);
            cCurrentTable = null;
        }

        //private String SoftHyphens(String pInput, int pMaxLength)
        //{
        //    String[] parts = pInput.Split(' ');
        //    StringBuilder result = new StringBuilder();
        //    for (int i = 0; i < parts.Length; i++)
        //    {
        //        if (i > 0) result.Append(" ");
        //        String sval = parts[i];
        //        while (sval.Length > pMaxLength)
        //        {
        //            result.Append(sval.Substring(0, pMaxLength));
        //            result.Append('\xad');
        //            sval = sval.Substring(pMaxLength);
        //        }
        //        result.Append(sval);
        //    }
        //    return result.ToString();
        //}

        private MDT.Table AddRow(params CellDefinition[] pItems)
        {
            int cols = 0;
            foreach (CellDefinition tid in pItems)
            {
                cols += tid.cWidthInCells;
            }

            System.Diagnostics.Debug.Assert(cCurrentTable != null, "Table not created!");
            System.Diagnostics.Debug.Assert(cols == cCurrentTable.Columns.Count, "Wrong number of cells");

            cCurrentRow = cCurrentTable.AddRow();
            int rowIndex = 0;

            foreach (CellDefinition item in pItems)
            {
                if (item.cWidthInCells > 1)
                {
                    cCurrentRow[rowIndex].MergeRight = item.cWidthInCells - 1;
                }

                MD.Paragraph paragraph = cCurrentRow[rowIndex].AddParagraph();

                MD.FormattedText fText;
                fText = paragraph.AddFormattedText(item.cText);
                fText.Bold = item.cFontBold;
                fText.Italic = item.cFontItalic;
                if (item.cFontUnderline)
                {
                    fText.Underline = MD.Underline.Single;
                }

                fText.Font.Size = item.cFont.Size;
                fText.Font.Name = item.cFont.Name;
                if (item.cBackgroundColor != MD.Colors.White)
                {
                    cCurrentRow[rowIndex].Shading.Color = item.cBackgroundColor;
                    fText.Bold = true;
                }

                if (item.cFontUnderline == true)
                {
                    fText.Underline = MD.Underline.Single;

                }
                else
                {
                    fText.Underline = MD.Underline.None;
                }

                fText.Color = item.cTextColor;
                cCurrentRow[rowIndex].Format.Alignment = item.cAllignment;

                rowIndex += item.cWidthInCells;
            }
            return cCurrentTable;
        }

        private TextColor GetColorFromValueAndTolerance(double pValue, double pToleranceGreen, double pToleranceYellow, double pToleranceOrange)
        {
            if (pValue < pToleranceGreen)
            {
                return new TextColor(MD.Colors.DarkGreen);
            }
            else if (pValue < pToleranceYellow)
            {
                return new TextColor(MD.Colors.Goldenrod);
            }
            else if (pValue < pToleranceOrange)
            {
                return new TextColor(MD.Colors.DarkOrange);
            }
            else
            {
                return new TextColor(MD.Colors.DarkRed);
            }
        }

        private string AddEllipses(string sInputString, MD.Font textFont, double cellWidth)
        {
            SizeF measuredTextSize;
            string modifiedString = string.Empty;
            char[] textArray;
            MD.TextMeasurement measure = new MD.TextMeasurement(textFont);
            measuredTextSize = measure.MeasureString(sInputString, MD.UnitType.Inch);

            if (cellWidth <= measuredTextSize.Width)
            {
                textArray = sInputString.ToCharArray();
                int length = sInputString.Length;
                for (int c = length; c >= 0; --c)
                {
                    modifiedString = sInputString.Substring(0, c);
                    for (int i = 0; i < 3; i++)
                    {
                        modifiedString += '\u2022';
                    }
                    measuredTextSize = measure.MeasureString(modifiedString, MD.UnitType.Inch);
                    if (measuredTextSize.Width + 0.15 < cellWidth) //Add some padding 
                    {
                        return modifiedString;
                    }
                }
            }
            if (modifiedString != string.Empty)
            {
                return modifiedString;
            }
            else
            {
                return sInputString;
            }
        }

        /// <summary>
        /// Sets the default style info (Font, pt size, etc.)
        /// </summary>
        private void SetDocumentStyles()
        {
            //Normal (Global base style)
            MD.Style style = cDocument.Styles["Normal"];
            style.Font.Name = ReportFont.cFontName;
            style.Font.Size = new MD.Unit(10, MD.UnitType.Point);

        }

        private String ScrubOwner(String pInput)
        {
            String sval = pInput.ToUpper().Trim();
            if (sval.StartsWith("<") && sval.EndsWith(">") && sval.Contains("UND")) return String.Empty;
            return pInput;
        }
    }
}
