﻿using System;
using System.Collections.Generic;
using System.Text;
using PPL_Lib;
using System.Windows.Forms;
using System.IO;
using System.Reflection;
using System.Reflection.Emit;
using System.Data;
using System.Data.SqlClient;

///----------------------------------------------------------------------------
/// This example plugin demonstrates
///   1) Adding a menu item to the main O-Calc Pro menu
///   2) Enabling and disabling that menu item
///   3) Performing a task when the item is clicked
///----------------------------------------------------------------------------

namespace OCalcProPlugin
{
    public class Plugin : PPLPluginInterface
    {

        /// <summary>
        /// THis is the handle to the main O-Calc Pro component
        /// </summary>
        PPL_Lib.PPLMain cPPLMain = null;

        /// <summary>
        /// Declare the type of plugin as one of:
        ///         DOCKED_TAB
        ///         MENU_ITEM
        ///         BOTH_DOCKED_AND_MENU
        ///         CLEARANCE_SAG_PROVIDER
        /// </summary>
        public PLUGIN_TYPE Type
        {
            get
            {
                return PLUGIN_TYPE.BOTH_DOCKED_AND_MENU;
            }
        }

        /// <summary>
        /// Declare the name of the plugin usd for synthesizing the registry keys ect
        /// </summary>
        public String Name
        {
            get
            {
                return "$safeprojectname$";
            }
        }

        /// <summary>
        /// Optionally declare a description string (defaults to the name);
        /// </summary>
        public String Description
        {
            get
            {
                return Name;
            }
        }

        /// <summary>
        /// Add a tabbed form to the tabbed window (if the plugin type is 
        /// PLUGIN_TYPE.DOCKED_TAB 
        /// or
        /// PLUGIN_TYPE.BOTH_DOCKED_AND_MENU
        /// </summary>
        /// <param name="pPPLMain"></param>
        public void AddForm(PPL_Lib.PPLMain pPPLMain)
        {
            cPPLMain = pPPLMain;
            cForm = new PluginForm();
            Guid guid = new Guid(0xeb496b8c, 0xed4a, 0x453e, 0xba, 0x32, 0x9, 0x7a, 0x49, 0x58, 0x47, 0x82);
            cForm.cGuid = guid;
            cPPLMain.cDockedPanels.Add(cForm.cGuid.ToString(), cForm);
            foreach (Control ctrl in cPPLMain.Controls)
            {
                if (ctrl is WeifenLuo.WinFormsUI.Docking.DockPanel)
                {
                    cForm.Show(ctrl as WeifenLuo.WinFormsUI.Docking.DockPanel, WeifenLuo.WinFormsUI.Docking.DockState.Document);
                }
            }
            cForm.Show();
        }

        PluginForm cForm = null;

        class PluginForm : WeifenLuo.WinFormsUI.Docking.DockContent
        {
            public PluginForm()
            {
                this.Name = "pluginForm";
                this.Text = "Plugin";
                System.Windows.Forms.Label helloLabel = new System.Windows.Forms.Label();
                helloLabel.AutoSize = true;
                helloLabel.Name = "helloLabel";
                helloLabel.Text = "Hello World";
                helloLabel.Location = new System.Drawing.Point(20, 20);
                helloLabel.Visible = true;
                Controls.Add(helloLabel);
            }
            public Guid cGuid;
            protected override string GetPersistString()
            {
                return cGuid.ToString();
            }
        }

        /// <summary>
        /// Perform clearance analysis if type is PLUGIN_TYPE.CLEARANCE_SAG_PROVIDER
        /// </summary>
        /// <param name="pMain"></param>
        /// <returns></returns>
        public PPLClearance.ClearanceSagProvider GetClearanceSagProvider(PPL_Lib.PPLMain pMain)
        {
            System.Diagnostics.Debug.Assert(Type == PLUGIN_TYPE.CLEARANCE_SAG_PROVIDER, Name + " is not a clearance provider plugin.");
            return null;
        }


        //the toolstrip item we will add to O-Calc Pro
        ToolStripMenuItem cToolStripMenuItemButton = null;

        public void AddToMenu(PPL_Lib.PPLMain pPPLMain, System.Windows.Forms.ToolStrip pToolStrip)
        {
            //save the reference to the O-Calc Pro main
            cPPLMain = pPPLMain;

            //create the toolstrip button
            cToolStripMenuItemButton = new ToolStripMenuItem(Name);
            cToolStripMenuItemButton.AutoToolTip = true;
            cToolStripMenuItemButton.ToolTipText = Description;

            //find the dropdown menu we want to add the toolsrip button to
            int itemindex = 0;
            System.Diagnostics.Debug.Assert(pToolStrip.Items[itemindex] is ToolStripDropDownButton);
            if (pToolStrip.Items[itemindex] is ToolStripDropDownButton)
            {
                ToolStripDropDownButton tsb = pToolStrip.Items[itemindex] as ToolStripDropDownButton;
                System.Diagnostics.Debug.Assert(tsb.Text == "&File");
                tsb.DropDownItems.Insert(tsb.DropDownItems.Count - 1, cToolStripMenuItemButton);
                tsb.DropDownItems.Insert(tsb.DropDownItems.Count - 1, new ToolStripSeparator());

                //add an event handler when the dropdown menu is opened to allow us
                //to enable or disble the toolstrip button (optional)
                tsb.DropDownOpened += tsb_DropDownOpened;
            }
            //add and event handler to detect the toolstrip button bein clicked by the user
            cToolStripMenuItemButton.Click += button_Click;
        }

        /// <summary>
        /// The menu containing our tool is being displayed.  Optionally
        /// enable or disable the toolstrip button depending on
        /// our criteria
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void tsb_DropDownOpened(object sender, EventArgs e)
        {
            bool enabled = false;
            if (cPPLMain != null)
            {
                enabled = (cPPLMain.GetMainStructure() is PPLPole);
            }
            cToolStripMenuItemButton.Enabled = enabled;
        }

        /// <summary>
        /// The menu item was clicked by the user
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void button_Click(object sender, EventArgs e)
        {
            try
            {
                DoPluginOperation();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }


        /// <summary>
        /// Perform the plugin tool's operation
        /// </summary>
        private void DoPluginOperation()
        {
            try
            {
                PPLMessageBox.Show("Hello World!");
            }
            catch (Exception ex)
            {
                PPLMessageBox.Show(ex.Message, "Error in " + Name);
            }
        }
    }
}
