﻿using System;
using System.Drawing;
using System.Collections.Generic;
using System.Collections;
using System.Collections.Specialized;
using System.Data;
using System.Data.SqlClient;
using System.Data.Common;
using System.Linq.Expressions;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using WeifenLuo.WinFormsUI.Docking;
using PPL_Lib;
using MigraDoc;
using MD = MigraDoc.DocumentObjectModel;
using MDT = MigraDoc.DocumentObjectModel.Tables;

namespace OCalcProReporting
{

    public static class ReportFont
    {
        public static double cPageWidth = 6.25;
        public static string cFontName = "Arial";
        public static MD.Unit cFontSizeLarge = MD.Unit.FromPoint(9.75);
        public static MD.Unit cFontSizeSmall = MD.Unit.FromPoint(7.85);
    }

    public partial class Report : PPL_Lib.ReportTemplate
    {

        //globally defined colors and fonts
        MD.Color Yellow = MD.Colors.LightYellow;
        MD.Color White = MD.Colors.White;
        MD.ParagraphAlignment JustifyRight = MD.ParagraphAlignment.Right;
        MD.ParagraphAlignment JustifyLeft = MD.ParagraphAlignment.Left;
        MD.ParagraphAlignment JustifyCenter = MD.ParagraphAlignment.Center;
        MD.Font SmallFont = new MD.Font(ReportFont.cFontName, ReportFont.cFontSizeSmall);
        MD.Font StandardFont = new MD.Font(ReportFont.cFontName, ReportFont.cFontSizeLarge);
        FontModifier Bold = new FontModifier(true, false, false);


        //implement more than one additional report subtype (total will be this number plus the master)
        public override int SubtypesCount { get { return 0; } }

        public override REPORT_TYPE Type { get { return REPORT_TYPE.INTEGRATED; } }
        public override String Name { get { return "<Name> Report"; } }
        public override String Description { get { return Name; } }

        //make sure to implement ToString to show report name
        public override String ToString() { return Name; }

        //determine if this report is applicable for a structure
        public override bool IsApplicable(PPLMain pPPLMain, PPLPole pPole)
        {
            if (Filtered) return false;
            if (pPole is PPLWoodPole)
            {
                return true;
            }
            return false;
        }
        public override bool IsApplicableForType(PPLMain.POLE_TYPE pType)
        {
            if (Filtered) return false;
            return (pType == PPLMain.POLE_TYPE.WOOD);
        }

        MigraDoc.DocumentObjectModel.Document cDocument = null;
        PPLMain cPPLMain = null;
        MDT.Table cCurrentTable = null;
        MDT.Row cCurrentRow = null;
        MD.Unit cMaxRowHeight = 0;

        //methods to create integrated report
        public override void CreateReport(PPLMain pPPLMain, List<FileInfo> pSelectedFiles)
        {
            cPPLMain = pPPLMain;
            cDocument = new MigraDoc.DocumentObjectModel.Document();
            MD.Section section = cPPLMain.BuildStandardReportHeader(cDocument, Name, true);
            cDocument.DefaultPageSetup.Orientation = MD.Orientation.Portrait;
            cDocument.LastSection.PageSetup.Orientation = MigraDoc.DocumentObjectModel.Orientation.Portrait;
            MigraDoc.DocumentObjectModel.Paragraph paragraph = null;
            paragraph = section.AddParagraph("\n");
            paragraph.Format.Borders.Top.Color = MigraDoc.DocumentObjectModel.Colors.Blue;
            paragraph.Format.Borders.Top.Width = new MigraDoc.DocumentObjectModel.Unit(0.025, MigraDoc.DocumentObjectModel.UnitType.Inch);
        }

        public override void AddPole(PPLPole pPole, String pPoleName)
        {
            String pplx = cPPLMain.ReportingLoadedPolePath;
            try
            {
                pplx = System.IO.Path.GetFileName(pplx);
            }
            catch { }


            PoleLoadInfo poleInfoMaster = cPPLMain.CurrentPoleLoadInfo;
            ReportItems reportItems = new ReportItems(poleInfoMaster, cPPLMain, pplx);
            ReportItems.ReportParameters RP = reportItems.cReportParameters;

            CreateTable(ReportFont.cPageWidth, 4);
            cCurrentTable.Format.Font = SmallFont.Clone();
            cCurrentTable.Borders.Visible = false;


            //Header Row
            AddRow(Cell("Pole Capacity Utilization (%)", 2, MD.Colors.White, JustifyLeft, StandardFont, Bold),
                    Cell("Height" + "\n(" + RP.Units_Len_Large_Abrev + ")", MD.Colors.White, JustifyRight, StandardFont, Bold),
                    Cell("Wind Angle" + "\n(deg)", MD.Colors.White, JustifyRight, StandardFont, Bold));
            cCurrentTable.RightPadding = MD.Unit.FromInch(0.125);

            //First Row
            AddRow(Cell("Maximum", JustifyLeft),
                  Cell(Math.Round(RP.Sum_MCU, 1).ToString("0.0"), Yellow, JustifyRight, StandardFont, Bold,
                          GetColorFromValueAndTolerance(RP.Sum_MCU, 80, 90, 99.5)),
                  Cell(Math.Round(RP.Sum_MCU_Height, 1).ToString("0.0"), JustifyRight),
                  Cell(Math.Round(RP.Sum_MCU_Wind_Angle, 1).ToString("0.0"), JustifyRight));

            //Second Row
            AddRow(Cell("Groundline", JustifyLeft),
                  Cell(Math.Round(RP.Sum_GLCU, 1).ToString("0.0"), Yellow, JustifyRight, StandardFont, Bold,
                          GetColorFromValueAndTolerance(RP.Sum_GLCU, 80, 90, 99.5)),
                  Cell(Math.Round(RP.Sum_GLCU_Height, 1).ToString("0.0"), JustifyRight),
                  Cell(Math.Round(RP.Sum_GLCU_Wind_Angle, 1).ToString("0.0"), JustifyRight));

            //Third Row
            AddRow( Cell("Vertical", JustifyLeft),
                    Cell(Math.Round(RP.Sum_Vertical_Buckling_CU, 1).ToString("0.0"), Yellow, JustifyRight, StandardFont, Bold,
                                        GetColorFromValueAndTolerance(RP.Sum_Vertical_Buckling_CU, 80, 90, 99.5)),
                    Cell(Math.Round(RP.Sum_Vertical_Buckling_Column_Height, 1).ToString("0.0"), JustifyRight),
                    Cell(Math.Round(RP.Sum_Vertical_Buckling_Wind_Angle, 1).ToString("0.0"), JustifyRight));

            cCurrentTable.BottomPadding = MD.Unit.FromInch(0.03);
            cCurrentTable.TopPadding = MD.Unit.FromInch(0.03);

            cCurrentTable.SetEdge(0, 0, cCurrentTable.Columns.Count, cCurrentTable.Rows.Count, MDT.Edge.Box, MD.BorderStyle.Single, 1, MD.Colors.Black);
            cCurrentTable.SetEdge(0, 0, cCurrentTable.Columns.Count, 1, MDT.Edge.Box, MD.BorderStyle.Single, 1, MD.Colors.Black);

            FinalizeTable();
            cDocument.LastSection.AddParagraph("\n");

        }

        public override MigraDoc.DocumentObjectModel.Document GetReport()
        {
            System.Diagnostics.Debug.Assert(Type == REPORT_TYPE.INTEGRATED || Type == REPORT_TYPE.BULK_SUMMARY);
            return cDocument;
        }

        //display standalone report
        public override void ShowReport(PPLMain pPPLMain, PPLPole pPole)
        {
            System.Diagnostics.Debug.Assert(Type == REPORT_TYPE.STANDALONE);
            throw new NotImplementedException("Report is not REPORT_TYPE.STANDALONE");
        }

        //housekeeping
        public override void Reset()
        {
            cPPLMain = null;
            cDocument = null;
        }



        private int cIndentStore = 0;
        private int cIndent
        {
            set
            {
                cIndentStore = Math.Max(0, value);
            }
            get
            {
                return cIndentStore;
            }
        }

        private String Indent()
        {
            return new String('\xA0', cIndentStore * 3);
        }

    }
}
